@@ -21,6 +21,7 @@
 #include "keyindex.h"
 #include "keystructs.h"
 #include "log.h"
+#include "onak-conf.h"
 
 int list_sigs(struct openpgp_packet_list *sigs, bool html)
 {
@@ -30,7 +31,7 @@ int list_sigs(struct openpgp_packet_list *sigs, bool html)
 
 	while (sigs != NULL) {
 		sigid = sig_keyid(sigs->packet);
-		uid = keyid2uid(sigid);
+		uid = config.dbbackend->keyid2uid(sigid);
 		if (sigs->packet->data[0] == 4 &&
 				sigs->packet->data[1] == 0x30) {
 			/* It's a Type 4 sig revocation */
@@ -40,21 +41,22 @@ int list_sigs(struct openpgp_packet_list *sigs, bool html)
 		}
 		if (html && uid != NULL) {
 			printf("%s         <a href=\"lookup?op=get&"
-				"search=%016llX\">%08llX</a>             "
-				"<a href=\"lookup?op=vindex&search=0x%016llX\">"
-				"%s</a>\n",
+				"search=%016" PRIX64 "\">%08" PRIX64
+				"</a>             "
+				"<a href=\"lookup?op=vindex&search=0x%016"
+				PRIX64 "\">%s</a>\n",
 				sig,
 				sigid,
 				sigid & 0xFFFFFFFF,
 				sigid,
 				txt2html(uid));
 		} else if (html && uid == NULL) {
-			printf("%s         %08llX             "
+			printf("%s         %08" PRIX64 "             "
 				"[User id not found]\n",
 				sig,
 				sigid & 0xFFFFFFFF);
 		} else {
-			printf("%s         %08llX"
+			printf("%s         %08" PRIX64
 				"             %s\n",
 				sig,
 				sigid & 0xFFFFFFFF,
@@ -87,9 +89,11 @@ int list_uids(uint64_t keyid, struct openpgp_signedpacket_list *uids,
 		} else if (uids->packet->tag == 17) {
 			printf("                                ");
 			if (html) {
-				printf("<img src=\"lookup?op=photo&search=0x%llX&idx=%d\" alt=\"[photo id]\">\n",
-						keyid,
-						imgindx);
+				printf("<img src=\"lookup?op=photo&search=0x%"
+					PRIX64 "&idx=%d\" alt=\"[photo id]\">"
+					"\n",
+					keyid,
+					imgindx);
 				imgindx++;
 			} else {
 				printf("[photo id]\n");
@@ -235,7 +239,8 @@ int key_index(struct openpgp_publickey *keys, bool verbose, bool fingerprint,
 
 		if (html) {
 			printf("pub  %5d%c/<a href=\"lookup?op=get&"
-				"search=%016llX\">%08llX</a> %04d/%02d/%02d ",
+				"search=%016" PRIX64 "\">%08" PRIX64
+				"</a> %04d/%02d/%02d ",
 				length,
 				(type == 1) ? 'R' : ((type == 16) ? 'g' : 
 					((type == 17) ? 'D' : '?')),
@@ -245,7 +250,7 @@ int key_index(struct openpgp_publickey *keys, bool verbose, bool fingerprint,
 				created->tm_mon + 1,
 				created->tm_mday);
 		} else {
-			printf("pub  %5d%c/%08llX %04d/%02d/%02d ",
+			printf("pub  %5d%c/%08" PRIX64 " %04d/%02d/%02d ",
 				length,
 				(type == 1) ? 'R' : ((type == 16) ? 'g' : 
 					((type == 17) ? 'D' : '?')),
@@ -262,7 +267,7 @@ int key_index(struct openpgp_publickey *keys, bool verbose, bool fingerprint,
 				curuid->packet->data);
 			if (html) {
 				printf("<a href=\"lookup?op=vindex&"
-					"search=0x%016llX\">",
+					"search=0x%016" PRIX64 "\">",
 					keyid);
 			}
 			printf("%s%s%s\n", 
@@ -327,7 +332,7 @@ int mrkey_index(struct openpgp_publickey *keys)
 		switch (keys->publickey->data[0]) {
 		case 2:
 		case 3:
-			printf("%016llX", get_keyid(keys));
+			printf("%016" PRIX64, get_keyid(keys));
 			type = keys->publickey->data[7];
 			length = (keys->publickey->data[8] << 8) +
 					keys->publickey->data[9];
