@@ -14,6 +14,7 @@
 #include "hash.h"
 #include "keydb.h"
 #include "ll.h"
+#include "onak-conf.h"
 #include "stats.h"
 
 /**
@@ -68,7 +69,7 @@ unsigned long findpath(struct stats_key *have, struct stats_key *want)
 	oldkeys = keys;
 
 	while ((!cleanup()) && keys != NULL && have->colour == 0) {
-		sigs = cached_getkeysigs(((struct stats_key *)
+		sigs = config.dbbackend->cached_getkeysigs(((struct stats_key *)
 					keys->object)->keyid);
 		while ((!cleanup()) && sigs != NULL && have->colour == 0) {
 			/*
@@ -128,21 +129,21 @@ void dofindpath(uint64_t have, uint64_t want, bool html, int count)
 	int pathnum;
 	char *uid;
 
-	fullhave = getfullkeyid(have);
-	fullwant = getfullkeyid(want);
+	fullhave = config.dbbackend->getfullkeyid(have);
+	fullwant = config.dbbackend->getfullkeyid(want);
 
 	/*
 	 * Make sure the keys we have and want are in the cache.
 	 */
-	(void) cached_getkeysigs(fullhave);
-	(void) cached_getkeysigs(fullwant);
+	(void) config.dbbackend->cached_getkeysigs(fullhave);
+	(void) config.dbbackend->cached_getkeysigs(fullwant);
 
 	if ((keyinfoa = findinhash(fullhave)) == NULL) {
-		printf("Couldn't find key 0x%llX.\n", have);
+		printf("Couldn't find key 0x%" PRIX64 ".\n", have);
 		return;
 	}
 	if ((keyinfob = findinhash(fullwant)) == NULL) {
-		printf("Couldn't find key 0x%llX.\n", want);
+		printf("Couldn't find key 0x%" PRIX64 ".\n", want);
 		return;
 	}
 
@@ -163,8 +164,8 @@ void dofindpath(uint64_t have, uint64_t want, bool html, int count)
 			html ? "<BR>" : "");
 		if (keyinfoa->colour == 0) {
 			if (pathnum == 0) {
-				printf("Can't find a link from 0x%08llX to "
-				"0x%08llX%s\n",
+				printf("Can't find a link from 0x%08" PRIX64
+				" to 0x%08" PRIX64 "%s\n",
 				have,
 				want,
 				html ? "<BR>" : "");
@@ -174,16 +175,19 @@ void dofindpath(uint64_t have, uint64_t want, bool html, int count)
 			}
 			pathnum = count;
 		} else {
-			printf("%d steps from 0x%08llX to 0x%08llX%s\n",
+			printf("%d steps from 0x%08" PRIX64 " to 0x%08" PRIX64
+				"%s\n",
 				keyinfoa->colour, have & 0xFFFFFFFF,
 				want & 0xFFFFFFFF,
 				html ? "<BR>" : "");
 			curkey = keyinfoa;
 			while (curkey != NULL && curkey->keyid != 0) {
-				uid = keyid2uid(curkey->keyid);
+				uid = config.dbbackend->keyid2uid(
+						curkey->keyid);
 				if (html && uid == NULL) {
 					printf("<a href=\"lookup?op=get&search="
-						"0x%08llX\">0x%08llX</a> (["
+						"0x%08" PRIX64 "\">0x%08" PRIX64
+						"</a> (["
 						"User id not found])%s<BR>\n",
 						curkey->keyid & 0xFFFFFFFF,
 						curkey->keyid & 0xFFFFFFFF,
@@ -191,9 +195,11 @@ void dofindpath(uint64_t have, uint64_t want, bool html, int count)
 							"" : " signs");
 				} else if (html && uid != NULL) {
 					printf("<a href=\"lookup?op=get&search="
-						"0x%08llX\">0x%08llX</a>"
+						"0x%08" PRIX64 "\">0x%08"
+						PRIX64 "</a>"
 						" (<a href=\"lookup?op=vindex&"
-						"search=0x%08llX\">%s</a>)%s"
+						"search=0x%08" PRIX64 
+						"\">%s</a>)%s"
 						"<BR>\n",
 						curkey->keyid & 0xFFFFFFFF,
 						curkey->keyid & 0xFFFFFFFF,
@@ -202,7 +208,7 @@ void dofindpath(uint64_t have, uint64_t want, bool html, int count)
 						(curkey->keyid == fullwant) ?
 						"" : " signs");
 				} else {
-					printf("0x%08llX (%s)%s\n",
+					printf("0x%08" PRIX64 " (%s)%s\n",
 						curkey->keyid & 0xFFFFFFFF,
 						(uid == NULL) ?
 							"[User id not found]" :
@@ -226,7 +232,8 @@ void dofindpath(uint64_t have, uint64_t want, bool html, int count)
 			}
 			curkey = keyinfoa;
 			while (curkey != NULL && curkey->keyid != 0) {
-				printf("0x%08llX ", curkey->keyid & 0xFFFFFFFF);
+				printf("0x%08" PRIX64 " ",
+						curkey->keyid & 0xFFFFFFFF);
 				curkey = findinhash(curkey->parent);
 			}
 			putchar('\n');
@@ -256,7 +263,7 @@ struct stats_key *furthestkey(struct stats_key *have)
 	curll = lladd(NULL, have);
 
 	while (curll != NULL) {
-		sigs = cached_getkeysigs(((struct stats_key *)
+		sigs = config.dbbackend->cached_getkeysigs(((struct stats_key *)
 				curll->object)->keyid);
 		while (sigs != NULL) {
 			if (((struct stats_key *) sigs->object)->colour == 0) {
